@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -83,8 +83,6 @@ zbx_graph_item_type;
 #	define TRIGGER_COMMENTS_LEN	65535
 #endif
 #define TRIGGER_EVENT_NAME_LEN		2048
-#define TAG_NAME_LEN			255
-#define TAG_VALUE_LEN			255
 
 #define GROUP_NAME_LEN			255
 
@@ -176,7 +174,9 @@ zbx_graph_item_type;
 #define ITEM_PARAMETER_NAME_LEN		255
 #define ITEM_PARAMETER_VALUE_LEN	2048
 
-#define ITEM_TAG_FIELD_LEN		255
+/* common tag/value field lengths for all tags */
+#define TAG_NAME_LEN			255
+#define TAG_VALUE_LEN			255
 
 #define HISTORY_STR_VALUE_LEN		255
 #define HISTORY_TEXT_VALUE_LEN		65535
@@ -262,6 +262,14 @@ zbx_graph_item_type;
 
 #define ZBX_DB_MAX_ID	(zbx_uint64_t)__UINT64_C(0x7fffffffffffffff)
 
+#ifdef HAVE_MYSQL
+#	define ZBX_SQL_SORT_ASC(field)	field " asc"
+#	define ZBX_SQL_SORT_DESC(field)	field " desc"
+#else
+#	define ZBX_SQL_SORT_ASC(field)	field " asc nulls first"
+#	define ZBX_SQL_SORT_DESC(field)	field " desc nulls last"
+#endif
+
 typedef struct
 {
 	zbx_uint64_t	druleid;
@@ -525,6 +533,7 @@ void	zbx_db_validate_config(void);
 
 #ifdef HAVE_ORACLE
 void	DBstatement_prepare(const char *sql);
+void	zbx_db_table_prepare(const char *tablename, struct zbx_json *json);
 #endif
 int		DBexecute(const char *fmt, ...) __zbx_attr_format_printf(1, 2);
 int		DBexecute_once(const char *fmt, ...) __zbx_attr_format_printf(1, 2);
@@ -540,14 +549,15 @@ int		DBend(int ret);
 
 const ZBX_TABLE	*DBget_table(const char *tablename);
 const ZBX_FIELD	*DBget_field(const ZBX_TABLE *table, const char *fieldname);
+int		zbx_db_validate_field_size(const char *tablename, const char *fieldname, const char *str);
 #define DBget_maxid(table)	DBget_maxid_num(table, 1)
 zbx_uint64_t	DBget_maxid_num(const char *tablename, int num);
 
-void	DBextract_version_info(struct zbx_db_version_info_t *version_info);
-void	DBflush_version_requirements(const char *version);
-int	DBcheck_capabilities(zbx_uint32_t db_version);
-
+void	zbx_db_extract_version_info(struct zbx_db_version_info_t *version_info);
+void	zbx_db_extract_dbextension_info(struct zbx_db_version_info_t *version_info);
+void	zbx_db_flush_version_requirements(const char *version);
 #ifdef HAVE_POSTGRESQL
+int	zbx_db_check_tsdb_capabilities(struct zbx_db_version_info_t *db_version_info, int allow_unsupported_ver);
 char	*zbx_db_get_schema_esc(void);
 #endif
 
@@ -687,6 +697,7 @@ int	DBtxn_ongoing(void);
 int	DBtable_exists(const char *table_name);
 int	DBfield_exists(const char *table_name, const char *field_name);
 #ifndef HAVE_SQLITE3
+int	DBtrigger_exists(const char *table_name, const char *trigger_name);
 int	DBindex_exists(const char *table_name, const char *index_name);
 int	DBpk_exists(const char *table_name);
 #endif
@@ -894,12 +905,41 @@ typedef struct
 }
 zbx_db_tag_t;
 
+ZBX_PTR_VECTOR_DECL(db_tag_ptr, zbx_db_tag_t *)
+
 zbx_db_tag_t	*zbx_db_tag_create(const char *tag_tag, const char *tag_value);
 void		zbx_db_tag_free(zbx_db_tag_t *tag);
-int		zbx_db_tag_compare_func(const void *d1, const void *d2);
-int		zbx_db_tag_compare_func_template(const void *d1, const void *d2);
 
-ZBX_PTR_VECTOR_DECL(db_tag_ptr, zbx_db_tag_t *)
+typedef struct _zbx_item_param_t zbx_item_param_t;
+struct _zbx_item_param_t
+{
+	zbx_uint64_t	item_parameterid;
+#define ZBX_FLAG_ITEM_PARAM_UPDATE_RESET	__UINT64_C(0x000000000000)
+#define ZBX_FLAG_ITEM_PARAM_UPDATE_NAME		__UINT64_C(0x000000000001)
+#define ZBX_FLAG_ITEM_PARAM_UPDATE_VALUE	__UINT64_C(0x000000000002)
+#define ZBX_FLAG_ITEM_PARAM_UPDATE			\
+		(ZBX_FLAG_ITEM_PARAM_UPDATE_NAME |	\
+		ZBX_FLAG_ITEM_PARAM_UPDATE_VALUE	\
+		)
+
+#define ZBX_FLAG_ITEM_PARAM_DELETE		__UINT64_C(0x000000010000)
+
+	zbx_uint64_t	flags;
+	char		*name_orig;
+	char		*name;
+	char		*value_orig;
+	char		*value;
+};
+
+ZBX_PTR_VECTOR_DECL(item_param_ptr, zbx_item_param_t *)
+
+zbx_item_param_t	*zbx_item_param_create(const char *item_param_name,
+		const char *item_param_value);
+void	zbx_item_param_free(zbx_item_param_t *param);
+
+
+int	zbx_merge_tags(zbx_vector_db_tag_ptr_t *dst, zbx_vector_db_tag_ptr_t *src, const char *owner, char **error);
+int	zbx_merge_item_params(zbx_vector_item_param_ptr_t *dst, zbx_vector_item_param_ptr_t *src, char **error);
 
 typedef enum
 {
@@ -945,4 +985,10 @@ void	zbx_db_trigger_get_itemids(const DB_TRIGGER *trigger, zbx_vector_uint64_t *
 
 int	DBselect_ids_names(const char *sql, zbx_vector_uint64_t *ids, zbx_vector_str_t *names);
 
+int	zbx_db_check_version_info(struct zbx_db_version_info_t *info, int allow_unsupported);
+
+#define ZBX_RECALC_TIME_PERIOD_HISTORY	1
+#define ZBX_RECALC_TIME_PERIOD_TRENDS	2
+void	zbx_recalc_time_period(int *ts_from, int table_group);
+
 #endif
