@@ -355,6 +355,11 @@
 #	include <pcre.h>
 #endif
 
+#ifdef HAVE_PCRE2_H
+#	define PCRE2_CODE_UNIT_WIDTH 8
+#	include <pcre2.h>
+#endif
+
 #ifdef HAVE_VM_VM_PARAM_H
 #	include <vm/vm_param.h>
 #endif
@@ -396,6 +401,15 @@
 #	else
 #		define ZBX_CURLOPT_ACCEPT_ENCODING	CURLOPT_ENCODING
 #	endif
+#	if 0x073700 <= LIBCURL_VERSION_NUM	/* version 7.55.0 */
+#		define ZBX_CURLINFO_SPEED_DOWNLOAD	CURLINFO_SPEED_DOWNLOAD_T
+#		define ZBX_CURLINFO_SPEED_DOWNLOAD_TYPE	curl_off_t
+#		define ZBX_CURLINFO_SPEED_DOWNLOAD_FMT	"%" CURL_FORMAT_CURL_OFF_T
+#	else
+#		define ZBX_CURLINFO_SPEED_DOWNLOAD	CURLINFO_SPEED_DOWNLOAD
+#		define ZBX_CURLINFO_SPEED_DOWNLOAD_TYPE	double
+#		define ZBX_CURLINFO_SPEED_DOWNLOAD_FMT	ZBX_FS_DBL
+#	endif
 #	define ZBX_CURLOPT_MAXREDIRS	10L
 #endif
 
