@@ -203,6 +203,7 @@ zbx_key_access_rule_type_t;
 void	init_metrics(void);
 int	add_metric(ZBX_METRIC *metric, char *error, size_t max_error_len);
 int	add_metric_local(ZBX_METRIC *metric, char *error, size_t max_error_len);
+void	free_metrics_ext(ZBX_METRIC **metrics);
 void	free_metrics(void);
 
 void	init_key_access_rules(void);
@@ -214,7 +215,11 @@ void	free_key_access_rules(void);
 
 int	process(const char *in_command, unsigned flags, AGENT_RESULT *result);
 
+void	set_user_parameter_dir(const char *path);
 int	add_user_parameter(const char *itemkey, char *command, char *error, size_t max_error_len);
+void	remove_user_parameters(void);
+void	get_metrics_copy(ZBX_METRIC **metrics);
+void	set_metrics(ZBX_METRIC *metrics);
 int	add_user_module(const char *key, int (*function)(void));
 void	test_parameters(void);
 void	test_parameter(const char *key);
@@ -243,6 +248,7 @@ zbx_uint64_t	get_kstat_numeric_value(const kstat_named_t *kn);
 int	GET_SENSOR(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	KERNEL_MAXFILES(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	KERNEL_MAXPROC(AGENT_REQUEST *request, AGENT_RESULT *result);
+int	KERNEL_OPENFILES(AGENT_REQUEST *request, AGENT_RESULT *result);
 
 #ifdef ZBX_PROCSTAT_COLLECTOR
 int	PROC_CPU_UTIL(AGENT_REQUEST *request, AGENT_RESULT *result);
@@ -256,7 +262,9 @@ int	NET_IF_TOTAL(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	NET_IF_COLLISIONS(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	NET_IF_DISCOVERY(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	NET_TCP_LISTEN(AGENT_REQUEST *request, AGENT_RESULT *result);
+int	NET_TCP_SOCKET_COUNT(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	NET_UDP_LISTEN(AGENT_REQUEST *request, AGENT_RESULT *result);
+int	NET_UDP_SOCKET_COUNT(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	SYSTEM_CPU_SWITCHES(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	SYSTEM_CPU_INTR(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	SYSTEM_CPU_LOAD(AGENT_REQUEST *request, AGENT_RESULT *result);
@@ -350,10 +358,12 @@ zbx_fsname_t;
 
 #define ZBX_LLD_MACRO_FSNAME		"{#FSNAME}"
 #define ZBX_LLD_MACRO_FSTYPE		"{#FSTYPE}"
+#define ZBX_LLD_MACRO_FSLABEL		"{#FSLABEL}"
 #define ZBX_LLD_MACRO_FSDRIVETYPE	"{#FSDRIVETYPE}"
 
 #define ZBX_SYSINFO_TAG_FSNAME			"fsname"
 #define ZBX_SYSINFO_TAG_FSTYPE			"fstype"
+#define ZBX_SYSINFO_TAG_FSLABEL			"fslabel"
 #define ZBX_SYSINFO_TAG_FSDRIVETYPE		"fsdrivetype"
 #define ZBX_SYSINFO_TAG_BYTES			"bytes"
 #define ZBX_SYSINFO_TAG_INODES			"inodes"
@@ -363,8 +373,24 @@ zbx_fsname_t;
 #define ZBX_SYSINFO_TAG_PFREE			"pfree"
 #define ZBX_SYSINFO_TAG_PUSED			"pused"
 
-int	zbx_execute_threaded_metric(zbx_metric_func_t metric_func, AGENT_REQUEST *request, AGENT_RESULT *result);
+#define ZBX_SYSINFO_FILE_TAG_TYPE		"type"
+#define ZBX_SYSINFO_FILE_TAG_BASENAME		"basename"
+#define ZBX_SYSINFO_FILE_TAG_PATHNAME		"pathname"
+#define ZBX_SYSINFO_FILE_TAG_DIRNAME		"dirname"
+#define ZBX_SYSINFO_FILE_TAG_USER		"user"
+#define ZBX_SYSINFO_FILE_TAG_GROUP		"group"
+#define ZBX_SYSINFO_FILE_TAG_PERMISSIONS	"permissions"
+#define ZBX_SYSINFO_FILE_TAG_SID		"SID"
+#define ZBX_SYSINFO_FILE_TAG_UID		"uid"
+#define ZBX_SYSINFO_FILE_TAG_GID		"gid"
+#define ZBX_SYSINFO_FILE_TAG_SIZE		"size"
+#define ZBX_SYSINFO_FILE_TAG_TIME		"time"
+#define ZBX_SYSINFO_FILE_TAG_TIMESTAMP		"timestamp"
+#define ZBX_SYSINFO_FILE_TAG_TIME_ACCESS	"access"
+#define ZBX_SYSINFO_FILE_TAG_TIME_MODIFY	"modify"
+#define ZBX_SYSINFO_FILE_TAG_TIME_CHANGE	"change"
 
+int	zbx_execute_threaded_metric(zbx_metric_func_t metric_func, AGENT_REQUEST *request, AGENT_RESULT *result);
 
 /* the fields used by proc queries */
 #define ZBX_SYSINFO_PROC_NONE		0x0000
