@@ -26,8 +26,8 @@ import (
 	"time"
 	"unicode"
 
+	"git.zabbix.com/ap/plugin-support/plugin"
 	"zabbix.com/pkg/itemutil"
-	"zabbix.com/pkg/plugin"
 	"zabbix.com/pkg/zbxcmd"
 )
 
@@ -41,6 +41,7 @@ type UserParameterPlugin struct {
 	plugin.Base
 	parameters           map[string]*parameterInfo
 	unsafeUserParameters int
+	userParameterDir     string
 }
 
 var userParameter UserParameterPlugin
@@ -105,7 +106,11 @@ func (p *UserParameterPlugin) cmd(key string, params []string) (string, error) {
 }
 
 // Export -
-func (p *UserParameterPlugin) Export(key string, params []string, ctx plugin.ContextProvider) (result interface{}, err error) {
+func (p *UserParameterPlugin) Export(
+	key string,
+	params []string,
+	ctx plugin.ContextProvider,
+) (result interface{}, err error) {
 	s, err := p.cmd(key, params)
 	if err != nil {
 		return nil, err
@@ -113,51 +118,99 @@ func (p *UserParameterPlugin) Export(key string, params []string, ctx plugin.Con
 
 	p.Debugf("executing command:'%s'", s)
 
-	stdoutStderr, err := zbxcmd.Execute(s, time.Second*time.Duration(Options.Timeout))
+	stdoutStderr, err := zbxcmd.Execute(s, time.Second*time.Duration(Options.Timeout), p.userParameterDir)
 	if err != nil {
 		return nil, err
 	}
 
-	p.Debugf("command:'%s' length:%d output:'%.20s'", s, len(stdoutStderr), stdoutStderr)
+	p.Debugf(
+		"command:'%s' length:%d output:'%.20s'",
+		s,
+		len(stdoutStderr),
+		stdoutStderr,
+	)
 
 	return stdoutStderr, nil
 }
 
-func InitUserParameterPlugin(userParameterConfig []string, unsafeUserParameters int) error {
-	userParameter.parameters = make(map[string]*parameterInfo)
-	userParameter.unsafeUserParameters = unsafeUserParameters
+func InitUserParameterPlugin(
+	userParameterConfig []string,
+	unsafeUserParameters int,
+	userParameterDir string,
+) ([]string, error) {
+	var (
+		keys   = make([]string, 0, len(userParameterConfig))
+		params = make(map[string]*parameterInfo)
+	)
 
-	for i := 0; i < len(userParameterConfig); i++ {
-		s := strings.SplitN(userParameterConfig[i], ",", 2)
-
-		if len(s) != 2 {
-			return fmt.Errorf("cannot add user parameter \"%s\": not comma-separated", userParameterConfig[i])
+	for _, userParam := range userParameterConfig {
+		// split by first comma.
+		parts := strings.SplitN(userParam, ",", 2) //nolint:gomnd
+		if len(parts) != 2 {                       //nolint:gomnd
+			return nil, fmt.Errorf(
+				"cannot add user parameter %q: not comma-separated", userParam,
+			)
 		}
 
-		key, p, err := itemutil.ParseKey(s[0])
+		key, keyParams, err := itemutil.ParseKey(parts[0])
 		if err != nil {
-			return fmt.Errorf("cannot add user parameter \"%s\": %s", userParameterConfig[i], err)
+			return nil, fmt.Errorf(
+				"cannot add user parameter %q: %s", userParam, err.Error(),
+			)
 		}
 
 		if acc, _ := plugin.Get(key); acc != nil {
-			return fmt.Errorf(`cannot register user parameter "%s": key already used`, userParameterConfig[i])
+			return nil, fmt.Errorf(
+				"cannot register user parameter %q: key already used",
+				userParam,
+			)
 		}
 
-		if len(strings.TrimSpace(s[1])) == 0 {
-			return fmt.Errorf("cannot add user parameter \"%s\": command is missing", userParameterConfig[i])
+		_, ok := params[key]
+		if ok {
+			return nil, fmt.Errorf(
+				"cannot register user parameter %q: duplicate user parameter",
+				userParam,
+			)
 		}
 
-		parameter := &parameterInfo{cmd: s[1]}
+		if len(strings.TrimSpace(parts[1])) == 0 {
+			return nil, fmt.Errorf(
+				"cannot add user parameter %q: command is missing", userParam,
+			)
+		}
 
-		if len(p) == 1 && p[0] == "*" {
+		parameter := &parameterInfo{cmd: parts[1]}
+
+		if len(keyParams) == 1 && keyParams[0] == "*" {
 			parameter.flexible = true
-		} else if len(p) != 0 {
-			return fmt.Errorf("cannot add user parameter \"%s\": syntax error", userParameterConfig[i])
 		}
 
-		userParameter.parameters[key] = parameter
-		plugin.RegisterMetrics(&userParameter, "UserParameter", key, fmt.Sprintf("User parameter: %s.", s[1]))
+		if len(keyParams) != 0 && !parameter.flexible {
+			return nil, fmt.Errorf(
+				"cannot add user parameter %q: syntax error", userParam,
+			)
+		}
+
+		params[key] = parameter
+		keys = append(keys, key)
 	}
 
-	return nil
+	for key, param := range params {
+		err := plugin.RegisterMetrics(
+			&userParameter,
+			"UserParameter",
+			key,
+			fmt.Sprintf("User parameter: %s.", param.cmd),
+		)
+		if err != nil {
+			return nil, err
+		}
+	}
+
+	userParameter.parameters = params
+	userParameter.unsafeUserParameters = unsafeUserParameters
+	userParameter.userParameterDir = userParameterDir
+
+	return keys, nil
 }
