@@ -27,6 +27,7 @@ package zbxlib
 #include "log.h"
 #include "../src/zabbix_agent/metrics.h"
 #include "../src/zabbix_agent/logfiles/logfiles.h"
+#include "../src/libs/zbxnix/fatal.h"
 
 extern int CONFIG_MAX_LINES_PER_SECOND;
 
@@ -64,7 +65,6 @@ void metric_get_meta(ZBX_ACTIVE_METRIC *metric, zbx_uint64_t *lastlogsize, int *
 void metric_set_unsupported(ZBX_ACTIVE_METRIC *metric)
 {
 	metric->state = ITEM_STATE_NOTSUPPORTED;
-	metric->refresh_unsupported = 0;
 	metric->error_count = 0;
 	metric->start_time = 0.0;
 	metric->processed_bytes = 0;
@@ -81,7 +81,6 @@ int metric_set_supported(ZBX_ACTIVE_METRIC *metric, zbx_uint64_t lastlogsize_sen
 		if (ITEM_STATE_NOTSUPPORTED == metric->state)
 		{
 			metric->state = ITEM_STATE_NORMAL;
-			metric->refresh_unsupported = 0;
 		}
 
 		if (lastlogsize_sent != metric->lastlogsize || mtime_sent != metric->mtime ||
@@ -178,12 +177,14 @@ static void free_log_result(log_result_t *result)
 	zbx_free(result);
 }
 
-int	process_value_cb(const char *server, unsigned short port, const char *host, const char *key,
+int	process_value_cb(zbx_vector_ptr_t *addrs, zbx_vector_ptr_t *agent2_result, const char *host, const char *key,
 		const char *value, unsigned char state, zbx_uint64_t *lastlogsize, const int *mtime,
 		unsigned long *timestamp, const char *source, unsigned short *severity, unsigned long *logeventid,
 		unsigned char flags)
 {
-	log_result_t *result = (log_result_t *)server;
+	ZBX_UNUSED(addrs);
+
+	log_result_t *result = (log_result_t *)agent2_result;
 	if (result->values.values_num == result->slots)
 		return FAIL;
 
@@ -191,21 +192,45 @@ int	process_value_cb(const char *server, unsigned short port, const char *host,
 	return SUCCEED;
 }
 
-static zbx_vector_pre_persistent_lp_t new_prep_vec(void)
-{
-	zbx_vector_pre_persistent_lp_t vect;
+#if !defined(__MINGW32__)
 
-	vect = (zbx_vector_pre_persistent_lp_t)zbx_malloc(NULL, sizeof(zbx_vector_pre_persistent_t));
-	zbx_vector_pre_persistent_create(vect);
-	return vect;
+static ZBX_THREAD_LOCAL struct sigaction sa_old;
+
+static void	fatal_signal_handler(int sig, siginfo_t *siginfo, void *context)
+{
+	zbx_log_fatal_info(context, ZBX_FATAL_LOG_FULL_INFO);
+
+	sigaction(SIGSEGV, &sa_old, NULL);
+	raise(sig);
 }
+#endif
 
-static void free_prep_vec(zbx_vector_pre_persistent_lp_t vect)
+static int	invoke_process_log_check(zbx_vector_ptr_t *agent2_result, zbx_vector_ptr_t *regexps,
+		ZBX_ACTIVE_METRIC *metric, zbx_process_value_func_t process_value_cb, zbx_uint64_t *lastlogsize_sent,
+		int *mtime_sent, char **error, zbx_uint64_t itemid)
 {
-	// In Agent2 this vector is expected to be empty because 'persistent directory' parameter is not allowed.
-	// Therefore a simplified cleanup is used.
-	zbx_vector_pre_persistent_destroy(vect);
-	zbx_free(vect);
+	int	ret;
+	zbx_vector_pre_persistent_t	vect;
+
+#if !defined(__MINGW32__)
+	struct sigaction	sa_new;
+	sigemptyset(&sa_new.sa_mask);
+	sa_new.sa_flags = SA_SIGINFO;
+	sa_new.sa_sigaction = fatal_signal_handler;
+	sigaction(SIGSEGV, &sa_new, &sa_old);
+#endif
+
+	zbx_vector_pre_persistent_create(&vect);
+
+	ret = process_log_check(NULL, agent2_result, regexps, metric, process_value_cb, lastlogsize_sent, mtime_sent, error,
+		&vect, itemid);
+
+#if !defined(__MINGW32__)
+	sigaction(SIGSEGV, &sa_old, NULL);
+#endif
+
+	zbx_vector_pre_persistent_destroy(&vect);
+	return ret;
 }
 */
 import "C"
@@ -215,8 +240,8 @@ import (
 	"time"
 	"unsafe"
 
+	"git.zabbix.com/ap/plugin-support/log"
 	"zabbix.com/pkg/itemutil"
-	"zabbix.com/pkg/log"
 )
 
 const (
@@ -275,6 +300,8 @@ func NewActiveMetric(key string, params []string, lastLogsize uint64, mtime int3
 	default:
 		return nil, errors.New("Unsupported item key.")
 	}
+
+	/* will be freed in FreeActiveMetric */
 	ckey := C.CString(itemutil.MakeKey(key, params))
 	log.Tracef("Calling C function \"new_metric()\"")
 	return unsafe.Pointer(C.new_metric(ckey, C.zbx_uint64_t(lastLogsize), C.int(mtime), C.int(flags))), nil
@@ -285,7 +312,7 @@ func FreeActiveMetric(data unsafe.Pointer) {
 	C.metric_free(C.ZBX_ACTIVE_METRIC_LP(data))
 }
 
-func ProcessLogCheck(data unsafe.Pointer, item *LogItem, refresh int, cblob unsafe.Pointer) {
+func ProcessLogCheck(data unsafe.Pointer, item *LogItem, refresh int, cblob unsafe.Pointer, itemid uint64) {
 	log.Tracef("Calling C function \"metric_set_refresh()\"")
 	C.metric_set_refresh(C.ZBX_ACTIVE_METRIC_LP(data), C.int(refresh))
 
@@ -300,14 +327,12 @@ func ProcessLogCheck(data unsafe.Pointer, item *LogItem, refresh int, cblob unsa
 	result := C.new_log_result(C.int(item.Output.PersistSlotsAvailable()))
 
 	var cerrmsg *C.char
-	log.Tracef("Calling C function \"new_prep_vec()\"")
-	cprepVec := C.new_prep_vec() // In Agent2 it is always empty vector. Not used but required for linking.
-	log.Tracef("Calling C function \"process_log_check()\"")
-	ret := C.process_log_check(C.char_lp_t(unsafe.Pointer(result)), 0, C.zbx_vector_ptr_lp_t(cblob),
+
+	log.Tracef("Calling C function \"invoke_process_log_check()\"")
+
+	ret := C.invoke_process_log_check(C.zbx_vector_ptr_lp_t(unsafe.Pointer(result)), C.zbx_vector_ptr_lp_t(cblob),
 		C.ZBX_ACTIVE_METRIC_LP(data), C.zbx_process_value_func_t(C.process_value_cb), &clastLogsizeSent,
-		&cmtimeSent, &cerrmsg, cprepVec)
-	log.Tracef("Calling C function \"free_prep_vec()\"")
-	C.free_prep_vec(cprepVec)
+		&cmtimeSent, &cerrmsg, C.zbx_uint64_t(itemid))
 
 	// add cached results
 	var cvalue *C.char
