@@ -20,8 +20,8 @@
 package mysql
 
 import (
-	"zabbix.com/pkg/conf"
-	"zabbix.com/pkg/plugin"
+	"git.zabbix.com/ap/plugin-support/conf"
+	"git.zabbix.com/ap/plugin-support/plugin"
 )
 
 // Session is a general structure for storing sessions' configuration.
@@ -51,6 +51,9 @@ type PluginOptions struct {
 	// Sessions stores pre-defined named sets of connections settings.
 	Sessions map[string]Session `conf:"optional"`
 
+	// CustomQueriesPath is a full pathname of a directory containing *.sql files with custom queries.
+	CustomQueriesPath string `conf:"optional"`
+
 	// Default stores default connection parameter values from configuration file
 	Default Session `conf:"optional"`
 }
