@@ -23,12 +23,16 @@ import (
 	"context"
 	"encoding/json"
 	"errors"
+	"fmt"
 
-	"zabbix.com/pkg/zbxerr"
+	"git.zabbix.com/ap/plugin-support/zbxerr"
 )
 
-func replicationSlaveStatusHandler(ctx context.Context, conn MyClient, _ map[string]string,
-	_ ...string) (interface{}, error) {
+const masterKey = "Master_Host"
+
+func replicationSlaveStatusHandler(
+	ctx context.Context, conn MyClient, params map[string]string, _ ...string,
+) (any, error) {
 	rows, err := conn.Query(ctx, `SHOW SLAVE STATUS`)
 	if err != nil {
 		return nil, zbxerr.ErrorCannotFetchData.Wrap(err)
@@ -43,7 +47,21 @@ func replicationSlaveStatusHandler(ctx context.Context, conn MyClient, _ map[str
 		return nil, zbxerr.ErrorEmptyResult.Wrap(errors.New("replication is not configured"))
 	}
 
-	jsonRes, err := json.Marshal(data[0])
+	if params[masterHostParam] != "" {
+		for _, m := range data {
+			if m[masterKey] == params[masterHostParam] {
+				return parseResponse(m)
+			}
+		}
+
+		return nil, zbxerr.ErrorEmptyResult.Wrap(fmt.Errorf("master host `%s` not found", params[masterHostParam]))
+	}
+
+	return parseResponse(data)
+}
+
+func parseResponse(data any) (any, error) {
+	jsonRes, err := json.Marshal(data)
 	if err != nil {
 		return nil, zbxerr.ErrorCannotMarshalJSON.Wrap(err)
 	}
