@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include "log.h"
 
 #include "zbxdb.h"
+#include "zbx_dbversion_constants.h"
 
 #if defined(HAVE_MYSQL)
 #	include "mysql.h"
@@ -64,21 +65,20 @@ struct zbx_db_result
 #endif
 };
 
-static int	txn_level = 0;	/* transaction level, nested transactions are not supported */
-static int	txn_error = ZBX_DB_OK;	/* failed transaction */
-static int	txn_end_error = ZBX_DB_OK;	/* transaction result */
+static int		txn_level = 0;	/* transaction level, nested transactions are not supported */
+static int		txn_error = ZBX_DB_OK;	/* failed transaction */
+static int		txn_end_error = ZBX_DB_OK;	/* transaction result */
 
-static char	*last_db_strerror = NULL;	/* last database error message */
+static char		*last_db_strerror = NULL;	/* last database error message */
 
-extern int	CONFIG_LOG_SLOW_QUERIES;
+extern int		CONFIG_LOG_SLOW_QUERIES;
 
-static int	db_auto_increment;
+static int		db_auto_increment;
 
 #if defined(HAVE_MYSQL)
 static MYSQL			*conn = NULL;
 static zbx_uint32_t		ZBX_MYSQL_SVERSION = ZBX_DBVERSION_UNDEFINED;
 static int			ZBX_MARIADB_SFORK = OFF;
-static char			*ZBX_CHARSET = NULL;
 #elif defined(HAVE_ORACLE)
 #include "zbxalgo.h"
 
@@ -104,15 +104,17 @@ static ub4	OCI_DBserver_status(void);
 #elif defined(HAVE_POSTGRESQL)
 static PGconn			*conn = NULL;
 static unsigned int		ZBX_PG_BYTEAOID = 0;
-static int			ZBX_TSDB_VERSION = -1;
+int			ZBX_TSDB_VERSION = -1;
 static zbx_uint32_t		ZBX_PG_SVERSION = ZBX_DBVERSION_UNDEFINED;
 char				ZBX_PG_ESCAPE_BACKSLASH = 1;
+static int 			ZBX_TIMESCALE_COMPRESSION_AVAILABLE = OFF;
 #elif defined(HAVE_SQLITE3)
 static sqlite3			*conn = NULL;
 static zbx_mutex_t		sqlite_access = ZBX_MUTEX_NULL;
 #endif
 
 #if defined(HAVE_ORACLE)
+static void	OCI_DBclean_result_handle(DB_RESULT result);
 static void	OCI_DBclean_result(DB_RESULT result);
 #endif
 
@@ -314,9 +316,12 @@ static DB_RESULT	zbx_db_select(const char *fmt, ...)
 }
 
 #if defined(HAVE_MYSQL)
-static int	is_recoverable_mysql_error(void)
+static int	is_recoverable_mysql_error(int err_no)
 {
-	switch (mysql_errno(conn))
+	if (0 == err_no)
+		err_no = (int)mysql_errno(conn);
+
+	switch (err_no)
 	{
 		case CR_CONN_HOST_ERROR:
 		case CR_SERVER_GONE_ERROR:
@@ -380,12 +385,12 @@ int	zbx_db_connect(char *host, char *user, char *password, char *dbname, char *d
 {
 	int		ret = ZBX_DB_OK, last_txn_error, last_txn_level;
 #if defined(HAVE_MYSQL)
-	const char	*charset;
 #if LIBMYSQL_VERSION_ID >= 80000	/* my_bool type is removed in MySQL 8.0 */
 	bool		mysql_reconnect = 1;
 #else
 	my_bool		mysql_reconnect = 1;
 #endif
+	int		err_no = 0;
 #elif defined(HAVE_ORACLE)
 	char		*connect = NULL;
 	sword		err = OCI_SUCCESS;
@@ -559,7 +564,8 @@ int	zbx_db_connect(char *host, char *user, char *password, char *dbname, char *d
 			NULL == mysql_real_connect(conn, host, user, password, dbname, port, dbsocket,
 				CLIENT_MULTI_STATEMENTS))
 	{
-		zbx_db_errlog(ERR_Z3001, mysql_errno(conn), mysql_error(conn), dbname);
+		err_no = (int)mysql_errno(conn);
+		zbx_db_errlog(ERR_Z3001, err_no, mysql_error(conn), dbname);
 		ret = ZBX_DB_FAIL;
 	}
 
@@ -572,24 +578,33 @@ int	zbx_db_connect(char *host, char *user, char *password, char *dbname, char *d
 	if (ZBX_DB_OK == ret && 0 != mysql_options(conn, MYSQL_OPT_RECONNECT, &mysql_reconnect))
 		zabbix_log(LOG_LEVEL_WARNING, "Cannot set MySQL reconnect option.");
 
-	charset = NULL == ZBX_CHARSET ? "utf8" : ZBX_CHARSET;
-	/* in contrast to "set names utf8" results of this call will survive auto-reconnects */
-	if (ZBX_DB_OK == ret && 0 != mysql_set_character_set(conn, charset))
-		zabbix_log(LOG_LEVEL_WARNING, "cannot set MySQL character set to \"%s\"", charset);
+	if (ZBX_DB_OK == ret)
+	{
+		/* in contrast to "set names utf8" results of this call will survive auto-reconnects */
+		/* utf8mb3 is deprecated and it's superset utf8mb4 should be used instead if available */
+		if (0 != mysql_set_character_set(conn, ZBX_SUPPORTED_DB_CHARACTER_SET_UTF8MB4) &&
+				0 != mysql_set_character_set(conn, ZBX_SUPPORTED_DB_CHARACTER_SET_UTF8) &&
+				0 != mysql_set_character_set(conn, ZBX_SUPPORTED_DB_CHARACTER_SET_UTF8MB3))
+		{
+			zabbix_log(LOG_LEVEL_WARNING, "cannot set MySQL character set");
+		}
+	}
 
 	if (ZBX_DB_OK == ret && 0 != mysql_autocommit(conn, 1))
 	{
-		zbx_db_errlog(ERR_Z3001, mysql_errno(conn), mysql_error(conn), dbname);
+		err_no = (int)mysql_errno(conn);
+		zbx_db_errlog(ERR_Z3001, err_no, mysql_error(conn), dbname);
 		ret = ZBX_DB_FAIL;
 	}
 
 	if (ZBX_DB_OK == ret && 0 != mysql_select_db(conn, dbname))
 	{
-		zbx_db_errlog(ERR_Z3001, mysql_errno(conn), mysql_error(conn), dbname);
+		err_no = (int)mysql_errno(conn);
+		zbx_db_errlog(ERR_Z3001, err_no, mysql_error(conn), dbname);
 		ret = ZBX_DB_FAIL;
 	}
 
-	if (ZBX_DB_FAIL == ret && SUCCEED == is_recoverable_mysql_error())
+	if (ZBX_DB_FAIL == ret && SUCCEED == is_recoverable_mysql_error(err_no))
 		ret = ZBX_DB_DOWN;
 
 #elif defined(HAVE_ORACLE)
@@ -871,6 +886,12 @@ out:
 	if (ZBX_DB_OK != ret)
 		goto out;
 
+	if (0 < (ret = zbx_db_execute("pragma foreign_keys=on")))
+		ret = ZBX_DB_OK;
+
+	if (ZBX_DB_OK != ret)
+		goto out;
+
 	if (0 < (ret = zbx_db_execute("pragma temp_store=2")))
 		ret = ZBX_DB_OK;
 
@@ -959,7 +980,7 @@ void	zbx_db_close(void)
 		for (i = 0; i < oracle.db_results.values_num; i++)
 		{
 			/* deallocate all handles before environment is deallocated */
-			OCI_DBclean_result(oracle.db_results.values[i]);
+			OCI_DBclean_result_handle(oracle.db_results.values[i]);
 		}
 	}
 
@@ -1420,9 +1441,9 @@ out:
  ******************************************************************************/
 int	zbx_db_vexecute(const char *fmt, va_list args)
 {
-	char	*sql = NULL;
-	int	ret = ZBX_DB_OK;
-	double	sec = 0;
+	char		*sql = NULL;
+	int		ret = ZBX_DB_OK;
+	double		sec = 0;
 #if defined(HAVE_MYSQL)
 #elif defined(HAVE_ORACLE)
 	sword		err = OCI_SUCCESS;
@@ -1461,13 +1482,15 @@ int	zbx_db_vexecute(const char *fmt, va_list args)
 	else
 	{
 		zbx_err_codes_t	errcode;
+		int		err_no;
 
 		if (0 != mysql_query(conn, sql))
 		{
-			errcode = (ER_DUP_ENTRY == mysql_errno(conn) ? ERR_Z3008 : ERR_Z3005);
-			zbx_db_errlog(errcode, mysql_errno(conn), mysql_error(conn), sql);
+			err_no = (int)mysql_errno(conn);
+			errcode = (ER_DUP_ENTRY == err_no ? ERR_Z3008 : ERR_Z3005);
+			zbx_db_errlog(errcode, err_no, mysql_error(conn), sql);
 
-			ret = (SUCCEED == is_recoverable_mysql_error() ? ZBX_DB_DOWN : ZBX_DB_FAIL);
+			ret = (SUCCEED == is_recoverable_mysql_error(err_no) ? ZBX_DB_DOWN : ZBX_DB_FAIL);
 		}
 		else
 		{
@@ -1486,10 +1509,11 @@ int	zbx_db_vexecute(const char *fmt, va_list args)
 				/* more results? 0 = yes (keep looping), -1 = no, >0 = error */
 				if (0 < (status = mysql_next_result(conn)))
 				{
-					errcode = (ER_DUP_ENTRY == mysql_errno(conn) ? ERR_Z3008 : ERR_Z3005);
-					zbx_db_errlog(errcode, mysql_errno(conn), mysql_error(conn), sql);
+					err_no = (int)mysql_errno(conn);
+					errcode = (ER_DUP_ENTRY == err_no ? ERR_Z3008 : ERR_Z3005);
+					zbx_db_errlog(errcode, err_no, mysql_error(conn), sql);
 
-					ret = (SUCCEED == is_recoverable_mysql_error() ? ZBX_DB_DOWN : ZBX_DB_FAIL);
+					ret = (SUCCEED == is_recoverable_mysql_error(err_no) ? ZBX_DB_DOWN : ZBX_DB_FAIL);
 				}
 			}
 			while (0 == status);
@@ -1642,10 +1666,13 @@ DB_RESULT	zbx_db_vselect(const char *fmt, va_list args)
 	{
 		if (0 != mysql_query(conn, sql) || NULL == (result->result = mysql_store_result(conn)))
 		{
-			zbx_db_errlog(ERR_Z3005, mysql_errno(conn), mysql_error(conn), sql);
+			int err_no;
+
+			err_no = (int)mysql_errno(conn);
+			zbx_db_errlog(ERR_Z3005, err_no, mysql_error(conn), sql);
 
 			DBfree_result(result);
-			result = (SUCCEED == is_recoverable_mysql_error() ? (DB_RESULT)ZBX_DB_DOWN : NULL);
+			result = (SUCCEED == is_recoverable_mysql_error(err_no) ? (DB_RESULT)ZBX_DB_DOWN : NULL);
 		}
 	}
 #elif defined(HAVE_ORACLE)
@@ -1762,9 +1789,16 @@ DB_RESULT	zbx_db_vselect(const char *fmt, va_list args)
 					}
 					else
 					{
+#define ZBX_MIN_OCI_NUMBER_WIDTH	22
 						/* retrieve the column width in bytes */
 						err = OCIAttrGet((void *)parmdp, (ub4)OCI_DTYPE_PARAM, (void *)&col_width,
 								(ub4 *)NULL, (ub4)OCI_ATTR_DATA_SIZE, (OCIError *)oracle.errhp);
+
+						if (ZBX_MIN_OCI_NUMBER_WIDTH > col_width && SQLT_NUM == data_type)
+						{
+							col_width = ZBX_MIN_OCI_NUMBER_WIDTH;
+						}
+#undef ZBX_MIN_OCI_NUMBER_WIDTH
 					}
 				}
 				col_width++;	/* add 1 byte for terminating '\0' */
@@ -2109,19 +2143,14 @@ int	zbx_db_is_null(const char *field)
 }
 
 #ifdef HAVE_ORACLE
-static void	OCI_DBclean_result(DB_RESULT result)
+static void	OCI_DBclean_result_handle(DB_RESULT result)
 {
-	if (NULL == result)
-		return;
-
 	if (NULL != result->values)
 	{
 		int	i;
 
 		for (i = 0; i < result->ncolumn; i++)
 		{
-			zbx_free(result->values[i]);
-
 			/* deallocate the lob locator variable */
 			if (NULL != result->clobs[i])
 			{
@@ -2129,10 +2158,6 @@ static void	OCI_DBclean_result(DB_RESULT result)
 				result->clobs[i] = NULL;
 			}
 		}
-
-		zbx_free(result->values);
-		zbx_free(result->clobs);
-		zbx_free(result->values_alloc);
 	}
 
 	if (result->stmthp)
@@ -2141,6 +2166,25 @@ static void	OCI_DBclean_result(DB_RESULT result)
 		result->stmthp = NULL;
 	}
 }
+static void	OCI_DBclean_result(DB_RESULT result)
+{
+	if (NULL == result)
+		return;
+
+	OCI_DBclean_result_handle(result);
+
+	if (NULL != result->values)
+	{
+		int	i;
+
+		for (i = 0; i < result->ncolumn; i++)
+			zbx_free(result->values[i]);
+
+		zbx_free(result->values);
+		zbx_free(result->clobs);
+		zbx_free(result->values_alloc);
+	}
+}
 #endif
 
 void	DBfree_result(DB_RESULT result)
@@ -2476,13 +2520,13 @@ int	zbx_db_version_check(const char *database, zbx_uint32_t current_version, zbx
 	else if (min_version > current_version && ZBX_DBVERSION_UNDEFINED != min_version)
 	{
 		flag = DB_VERSION_LOWER_THAN_MINIMUM;
-		zabbix_log(LOG_LEVEL_WARNING, "Unsupported DB! %s version is %lu which is smaller than minimum of %lu",
+		zabbix_log(LOG_LEVEL_WARNING, "Unsupported DB! %s version %lu is older than minimum required %lu",
 				database, (unsigned long)current_version, (unsigned long)min_version);
 	}
 	else if (max_version < current_version && ZBX_DBVERSION_UNDEFINED != max_version)
 	{
 		flag = DB_VERSION_HIGHER_THAN_MAXIMUM;
-		zabbix_log(LOG_LEVEL_WARNING, "Unsupported DB! %s version is %lu which is higher than maximum of %lu",
+		zabbix_log(LOG_LEVEL_WARNING, "Unsupported DB! %s version %lu is newer than maximum allowed %lu",
 				database, (unsigned long)current_version, (unsigned long)max_version);
 	}
 	else if (min_supported_version > current_version && ZBX_DBVERSION_UNDEFINED != min_supported_version)
@@ -2500,7 +2544,8 @@ int	zbx_db_version_check(const char *database, zbx_uint32_t current_version, zbx
  *                                                                            *
  * Purpose: prepare json for front-end with the DB current, minimum and       *
  *          maximum versions and a flag that indicates if the version         *
- *          satisfies the requirements                                        *
+ *          satisfies the requirements, and other information as well as      *
+ *          information about DB extension in a similar way                   *
  *                                                                            *
  * Parameters:  json                     - [IN/OUT] json data                 *
  *              info                     - [IN] info to serialize             *
@@ -2525,7 +2570,53 @@ void	zbx_db_version_json_create(struct zbx_json *json, struct zbx_db_version_inf
 	}
 
 	zbx_json_addint64(json, "flag", info->flag);
+#ifdef HAVE_ORACLE
+	if (0 != info->tables_json.buffer_offset)
+	{
+		zbx_json_addobject(json, "schema_diff");
+		if (0 != strcmp(info->tables_json.buffer, "{}"))
+			zbx_json_addraw(json, "tables", info->tables_json.buffer);
+		zbx_json_close(json);
+	}
+#endif
 	zbx_json_close(json);
+
+	if (NULL != info->extension)
+	{
+		zbx_json_addobject(json, NULL);
+		zbx_json_addstring(json, "database", info->extension, ZBX_JSON_TYPE_STRING);
+
+		if (DB_VERSION_FAILED_TO_RETRIEVE != info->ext_flag)
+		{
+			zbx_json_addstring(json, "current_version",
+					info->ext_friendly_current_version, ZBX_JSON_TYPE_STRING);
+		}
+
+		zbx_json_addstring(json, "min_version", info->ext_friendly_min_version, ZBX_JSON_TYPE_STRING);
+		zbx_json_addstring(json, "max_version", info->ext_friendly_max_version, ZBX_JSON_TYPE_STRING);
+		zbx_json_addstring(json, "min_supported_version", info->ext_friendly_min_supported_version,
+				ZBX_JSON_TYPE_STRING);
+
+		zbx_json_addint64(json, "flag", info->ext_flag);
+		zbx_json_addint64(json, "extension_err_code", info->ext_err_code);
+#ifdef HAVE_POSTGRESQL
+		if (0 == zbx_strcmp_null(info->extension, ZBX_DB_EXTENSION_TIMESCALEDB))
+		{
+			if (ON == zbx_tsdb_get_compression_availability())
+			{
+				zbx_json_addstring(json, "compression_availability", "true", ZBX_JSON_TYPE_INT);
+			}
+			else
+			{
+				zbx_json_addstring(json, "compression_availability", "false", ZBX_JSON_TYPE_INT);
+			}
+
+			zbx_json_addint64(json, ZBX_TIMESCALE_COMPRESSED_CHUNKS_HISTORY, info->history_compressed_chunks);
+			zbx_json_addint64(json, ZBX_TIMESCALE_COMPRESSED_CHUNKS_TRENDS, info->trends_compressed_chunks);
+		}
+#endif
+		zbx_json_close(json);
+	}
 }
 
 /******************************************************************************
@@ -2642,13 +2733,13 @@ void	zbx_dbms_version_info_extract(struct zbx_db_version_info_t *version_info)
 	{
 		version_info->database = "MariaDB";
 
-		version_info->min_version = ZBX_MARIA_MIN_VERSION;
-		version_info->max_version = ZBX_MARIA_MAX_VERSION;
-		version_info->min_supported_version = ZBX_MARIA_MIN_SUPPORTED_VERSION;
+		version_info->min_version = ZBX_MARIADB_MIN_VERSION;
+		version_info->max_version = ZBX_MARIADB_MAX_VERSION;
+		version_info->min_supported_version = ZBX_MARIADB_MIN_SUPPORTED_VERSION;
 
-		version_info->friendly_min_version = ZBX_MARIA_MIN_VERSION_FRIENDLY;
-		version_info->friendly_max_version = ZBX_MARIA_MAX_VERSION_FRIENDLY;
-		version_info->friendly_min_supported_version = ZBX_MARIA_MIN_SUPPORTED_VERSION_FRIENDLY;
+		version_info->friendly_min_version = ZBX_MARIADB_MIN_VERSION_STR;
+		version_info->friendly_max_version = ZBX_MARIADB_MAX_VERSION_STR;
+		version_info->friendly_min_supported_version = ZBX_MARIADB_MIN_SUPPORTED_VERSION_STR;
 	}
 	else
 	{
@@ -2658,9 +2749,9 @@ void	zbx_dbms_version_info_extract(struct zbx_db_version_info_t *version_info)
 		version_info->max_version = ZBX_MYSQL_MAX_VERSION;
 		version_info->min_supported_version = ZBX_MYSQL_MIN_SUPPORTED_VERSION;
 
-		version_info->friendly_min_version = ZBX_MYSQL_MIN_VERSION_FRIENDLY;
-		version_info->friendly_max_version = ZBX_MYSQL_MAX_VERSION_FRIENDLY;
-		version_info->friendly_min_supported_version = ZBX_MYSQL_MIN_SUPPORTED_VERSION_FRIENDLY;
+		version_info->friendly_min_version = ZBX_MYSQL_MIN_VERSION_STR;
+		version_info->friendly_max_version = ZBX_MYSQL_MAX_VERSION_STR;
+		version_info->friendly_min_supported_version = ZBX_MYSQL_MIN_SUPPORTED_VERSION_STR;
 	}
 
 	version_info->flag = zbx_db_version_check(version_info->database, version_info->current_version,
@@ -2670,9 +2761,9 @@ void	zbx_dbms_version_info_extract(struct zbx_db_version_info_t *version_info)
 	zbx_uint32_t major;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
-	ZBX_PG_SVERSION = PQserverVersion(conn);
+	ZBX_PG_SVERSION = (zbx_uint32_t)PQserverVersion(conn);
 
-	major = RIGHT2(ZBX_PG_SVERSION/10000);
+	major = ZBX_PG_SVERSION/10000;
 
 	version_info->database = "PostgreSQL";
 
@@ -2683,17 +2774,18 @@ void	zbx_dbms_version_info_extract(struct zbx_db_version_info_t *version_info)
 
 	if (10 > major)
 	{
-		version_info->friendly_current_version = zbx_dsprintf(NULL, "%d.%d.%d", major,
+		version_info->friendly_current_version = zbx_dsprintf(NULL, "%" PRIu32 ".%d.%d", major,
 				RIGHT2(ZBX_PG_SVERSION/100), RIGHT2(ZBX_PG_SVERSION));
 	}
 	else
 	{
-		version_info->friendly_current_version = zbx_dsprintf(NULL, "%d.%d", major, RIGHT2(ZBX_PG_SVERSION));
+		version_info->friendly_current_version = zbx_dsprintf(NULL, "%" PRIu32 ".%d", major,
+				RIGHT2(ZBX_PG_SVERSION));
 	}
 
-	version_info->friendly_min_version = ZBX_POSTGRESQL_MIN_VERSION_FRIENDLY;
-	version_info->friendly_max_version = ZBX_POSTGRESQL_MAX_VERSION_FRIENDLY;
-	version_info->friendly_min_supported_version = ZBX_POSTGRESQL_MIN_SUPPORTED_VERSION_FRIENDLY;
+	version_info->friendly_min_version = ZBX_POSTGRESQL_MIN_VERSION_STR;
+	version_info->friendly_max_version = ZBX_POSTGRESQL_MAX_VERSION_STR;
+	version_info->friendly_min_supported_version = ZBX_POSTGRESQL_MIN_SUPPORTED_VERSION_STR;
 
 	version_info->flag = zbx_db_version_check(version_info->database, version_info->current_version,
 			version_info->min_version, version_info->max_version, version_info->min_supported_version);
@@ -2779,9 +2871,9 @@ out:
 	version_info->min_supported_version = ZBX_ORACLE_MIN_SUPPORTED_VERSION;
 
 	version_info->friendly_current_version = zbx_strdup(NULL, version_friendly);
-	version_info->friendly_min_version = ZBX_ORACLE_MIN_VERSION_FRIENDLY;
-	version_info->friendly_max_version = ZBX_ORACLE_MAX_VERSION_FRIENDLY;
-	version_info->friendly_min_supported_version = ZBX_ORACLE_MIN_SUPPORTED_VERSION_FRIENDLY;
+	version_info->friendly_min_version = ZBX_ORACLE_MIN_VERSION_STR;
+	version_info->friendly_max_version = ZBX_ORACLE_MAX_VERSION_STR;
+	version_info->friendly_min_supported_version = ZBX_ORACLE_MIN_SUPPORTED_VERSION_STR;
 
 	version_info->flag = zbx_db_version_check(version_info->database, version_info->current_version,
 			version_info->min_version, version_info->max_version, version_info->min_supported_version);
@@ -2794,7 +2886,44 @@ out:
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s() version:%lu", __func__, (unsigned long)zbx_dbms_version_get());
 }
 
-#if defined(HAVE_POSTGRESQL)
+#ifdef HAVE_POSTGRESQL
+/***************************************************************************************************************
+ *                                                                                                             *
+ * Purpose: retrieves TimescaleDB extension info, including license string and numeric version value           *
+ *                                                                                                             *
+ **************************************************************************************************************/
+void	zbx_tsdb_info_extract(struct zbx_db_version_info_t *version_info)
+{
+	int	tsdb_ver;
+
+	if (0 != zbx_strcmp_null(version_info->extension, ZBX_DB_EXTENSION_TIMESCALEDB))
+		return;
+
+	tsdb_ver = zbx_tsdb_get_version();
+
+	version_info->ext_current_version = (zbx_uint32_t)tsdb_ver;
+	version_info->ext_min_version = ZBX_TIMESCALE_MIN_VERSION;
+	version_info->ext_max_version = ZBX_TIMESCALE_MAX_VERSION;
+	version_info->ext_min_supported_version = ZBX_TIMESCALE_MIN_SUPPORTED_VERSION;
+
+	version_info->ext_friendly_current_version = zbx_dsprintf(NULL, "%d.%d.%d", RIGHT2(tsdb_ver/10000),
+			RIGHT2(tsdb_ver/100), RIGHT2(tsdb_ver));
+
+	version_info->ext_friendly_min_version = ZBX_TIMESCALE_MIN_VERSION_STR;
+	version_info->ext_friendly_max_version = ZBX_TIMESCALE_MAX_VERSION_STR;
+	version_info->ext_friendly_min_supported_version = ZBX_TIMESCALE_MIN_SUPPORTED_VERSION_STR;
+
+	version_info->ext_flag = zbx_db_version_check(version_info->extension, version_info->ext_current_version,
+			version_info->ext_min_version, version_info->ext_max_version,
+			version_info->ext_min_supported_version);
+
+	if (ZBX_TIMESCALE_MIN_VERSION_WITH_LICENSE_PARAM_SUPPORT <= tsdb_ver)
+		version_info->ext_lic = zbx_tsdb_get_license();
+
+	zabbix_log(LOG_LEVEL_DEBUG, "TimescaleDB version: [%d], license: [%s]", tsdb_ver,
+		ZBX_NULL2EMPTY_STR(version_info->ext_lic));
+}
+
 /******************************************************************************
  *                                                                            *
  * Purpose: returns TimescaleDB (TSDB) version as integer: MMmmuu             *
@@ -2857,11 +2986,60 @@ out:
 	return ver;
 }
 
-#endif
-
-#if defined(HAVE_MYSQL)
-void zbx_db_set_character_set(const char *char_set)
+/******************************************************************************
+ *                                                                            *
+ * Purpose: retrievs TimescaleDB (TSDB) license information                   *
+ *                                                                            *
+ * Return value: license information from datase as string                    *
+ *               "apache"    for TimescaleDB Apache 2 Edition                 *
+ *               "timescale" for TimescaleDB Community Edition                *
+ *                                                                            *
+ * Comments: returns a pointer to allocated memory                            *
+ *                                                                            *
+ ******************************************************************************/
+char	*zbx_tsdb_get_license(void)
 {
-	ZBX_CHARSET = zbx_strdup(ZBX_CHARSET, char_set);
+	DB_RESULT	result;
+	DB_ROW		row;
+	char		*tsdb_lic = NULL;
+
+	result = zbx_db_select("show timescaledb.license");
+
+	if ((DB_RESULT)ZBX_DB_DOWN != result && NULL != result && NULL != (row = zbx_db_fetch(result)))
+	{
+		tsdb_lic = zbx_strdup(NULL, row[0]);
+	}
+
+	DBfree_result(result);
+
+	return tsdb_lic;
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Purpose: sets TimescaleDB (TSDB) compression availability                  *
+ *                                                                            *
+ * Parameters:  compression_availabile - [IN] compression availability        *
+ *              0 (OFF): compression is not available                         *
+ *              1 (ON): compression is available                              *
+ *                                                                            *
+ ******************************************************************************/
+void	zbx_tsdb_set_compression_availability(int compression_availabile)
+{
+	ZBX_TIMESCALE_COMPRESSION_AVAILABLE = compression_availabile;
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Purpose: retrievs TimescaleDB (TSDB) compression availability              *
+ *                                                                            *
+ * Return value: compression availability as as integer                       *
+ *               0 (OFF): compression is not available                        *
+ *               1 (ON): compression is available                             *
+ *                                                                            *
+ ******************************************************************************/
+int	zbx_tsdb_get_compression_availability(void)
+{
+	return ZBX_TIMESCALE_COMPRESSION_AVAILABLE;
 }
 #endif
