@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -982,7 +982,7 @@ static int	DBpatch_snmp_if_save(zbx_vector_dbu_snmp_if_t *snmp_ifs)
 
 	zbx_db_insert_prepare(&db_insert_snmp_if, "interface_snmp", "interfaceid", "version", "bulk", "community",
 			"securityname", "securitylevel", "authpassphrase", "privpassphrase", "authprotocol",
-			"privprotocol", "contextname", NULL);
+			"privprotocol", "contextname", (char *)NULL);
 
 	for (i = 0; i < snmp_ifs->values_num; i++)
 	{
@@ -1008,7 +1008,7 @@ static int	DBpatch_interface_create(zbx_vector_dbu_interface_t *interfaces)
 	int		i, ret;
 
 	zbx_db_insert_prepare(&db_insert_interfaces, "interface", "interfaceid", "hostid", "main", "type", "useip",
-			"ip", "dns", "port", NULL);
+			"ip", "dns", "port", (char *)NULL);
 
 	for (i = 0; i < interfaces->values_num; i++)
 	{
@@ -1369,7 +1369,7 @@ static int	DBpatch_interface_discovery_save(zbx_vector_uint64_pair_t *if_links)
 	zbx_db_insert_t	db_insert_if_links;
 	int		i, ret;
 
-	zbx_db_insert_prepare(&db_insert_if_links, "interface_discovery", "interfaceid", "parent_interfaceid", NULL);
+	zbx_db_insert_prepare(&db_insert_if_links, "interface_discovery", "interfaceid", "parent_interfaceid", (char *)NULL);
 
 	for (i = 0; i < if_links->values_num; i++)
 	{
