@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -145,9 +145,9 @@ static int	DBpatch_5050009(void)
 			"on t.triggerid=s.triggerid order by t.triggerid");
 
 	zbx_db_insert_prepare(&ins_service_problem_tag, "service_problem_tag", "service_problem_tagid", "serviceid",
-			"tag", "operator", "value", NULL);
-	zbx_db_insert_prepare(&ins_trigger_tag, "trigger_tag", "triggertagid", "triggerid", "tag", "value", NULL);
-	zbx_db_insert_prepare(&ins_problem_tag, "problem_tag", "problemtagid", "eventid", "tag", "value", NULL);
+			"tag", "operator", "value", (char *)NULL);
+	zbx_db_insert_prepare(&ins_trigger_tag, "trigger_tag", "triggertagid", "triggerid", "tag", "value", (char *)NULL);
+	zbx_db_insert_prepare(&ins_problem_tag, "problem_tag", "problemtagid", "eventid", "tag", "value", (char *)NULL);
 
 	while (NULL != (row = DBfetch(result)))
 	{
@@ -336,28 +336,6 @@ static int	DBpatch_5050031(void)
 	return DBadd_field("config", &field);
 }
 
-static int	DBpatch_5050032(void)
-{
-	if (0 == (program_type & ZBX_PROGRAM_TYPE_SERVER))
-		return SUCCEED;
-
-	if (ZBX_DB_OK > DBexecute("update config set default_lang='en_US' where default_lang='en_GB'"))
-		return FAIL;
-
-	return SUCCEED;
-}
-
-static int	DBpatch_5050033(void)
-{
-	if (0 == (program_type & ZBX_PROGRAM_TYPE_SERVER))
-		return SUCCEED;
-
-	if (ZBX_DB_OK > DBexecute("update users set lang='en_US' where lang='en_GB'"))
-		return FAIL;
-
-	return SUCCEED;
-}
-
 static int	DBpatch_5050034(void)
 {
 	const ZBX_FIELD	field = {"default_lang", "en_US", NULL, NULL, 5, ZBX_TYPE_CHAR, ZBX_NOTNULL, 0};
@@ -705,7 +683,7 @@ static int	DBpatch_5050068(void)
 	if (0 == (program_type & ZBX_PROGRAM_TYPE_SERVER))
 		return SUCCEED;
 
-	zbx_db_insert_prepare(&db_insert, "role_rule", "role_ruleid", "roleid", "type", "name", "value_int", NULL);
+	zbx_db_insert_prepare(&db_insert, "role_rule", "role_ruleid", "roleid", "type", "name", "value_int", (char *)NULL);
 
 	result = DBselect("select roleid,type from role");
 
@@ -1476,18 +1454,18 @@ static int	db_insert_sla(const zbx_vector_sla_t *uniq_slas, const char *default_
 	int			ret = FAIL;
 
 	zbx_db_insert_prepare(&db_insert_sla, "sla", "slaid", "name", "status", "slo", "effective_date", "period",
-			"timezone", NULL);
+			"timezone", (char *)NULL);
 
 	zbx_db_insert_prepare(&db_insert_sla_service_tag, "sla_service_tag", "sla_service_tagid", "slaid", "tag",
-			"value", NULL);
+			"value", (char *)NULL);
 
 	zbx_db_insert_prepare(&db_insert_service_tag, "service_tag", "servicetagid", "serviceid", "tag", "value",
-			NULL);
+			(char *)NULL);
 
 	zbx_db_insert_prepare(&db_insert_sla_schedule, "sla_schedule", "sla_scheduleid", "slaid", "period_from",
-			"period_to", NULL);
+			"period_to", (char *)NULL);
 	zbx_db_insert_prepare(&db_insert_sla_excluded_downtime, "sla_excluded_downtime", "sla_excluded_downtimeid",
-			"slaid", "period_from", "period_to", "name", NULL);
+			"slaid", "period_from", "period_to", "name", (char *)NULL);
 
 	for (i = 0, slaid = 0; i < uniq_slas->values_num; i++)
 	{
@@ -1590,6 +1568,9 @@ static void	services_times_convert_downtime(zbx_vector_services_times_t *service
 	{
 		services_times_t	*service_downtime = &services_downtimes.values[i];
 
+		if (service_downtime->from > service_downtime->to)
+			continue;
+
 		for (j = 0; j < services_times->values_num; j++)
 		{
 			services_times_t	*service_time = &services_times->values[j];
@@ -1611,6 +1592,7 @@ static void	services_times_convert_downtime(zbx_vector_services_times_t *service
 						service_time_new.note = zbx_strdup(NULL, "");
 
 						zbx_vector_services_times_append(services_times, service_time_new);
+						service_time = &services_times->values[j];
 					}
 
 					service_time->to = service_downtime->from;
@@ -1985,8 +1967,6 @@ DBPATCH_ADD(5050023, 0, 1)
 DBPATCH_ADD(5050024, 0, 1)
 DBPATCH_ADD(5050030, 0, 1)
 DBPATCH_ADD(5050031, 0, 1)
-DBPATCH_ADD(5050032, 0, 1)
-DBPATCH_ADD(5050033, 0, 1)
 DBPATCH_ADD(5050034, 0, 1)
 DBPATCH_ADD(5050040, 0, 1)
 DBPATCH_ADD(5050041, 0, 1)
