@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -223,12 +223,14 @@ static int	eval_parse_constant(zbx_eval_context_t *ctx, size_t pos, zbx_eval_tok
 {
 	zbx_token_t		tok;
 	size_t			offset = pos;
-	zbx_token_type_t	type = 0;
+	zbx_token_type_t	type = 0, last_type = 0;
 
 	do
 	{
 		if ('{' == (ctx->expression[offset]))
 		{
+			last_type = ZBX_TOKEN_MACRO;
+
 			if (SUCCEED != eval_parse_macro(ctx, (int)offset, &tok))
 				break;
 
@@ -271,9 +273,9 @@ static int	eval_parse_constant(zbx_eval_context_t *ctx, size_t pos, zbx_eval_tok
 					goto out;
 			}
 		}
-		else if (SUCCEED == eval_parse_number(ctx, offset, &offset))
+		else if (ZBX_EVAL_TOKEN_VAR_NUM != last_type && SUCCEED == eval_parse_number(ctx, offset, &offset))
 		{
-			type = ZBX_EVAL_TOKEN_VAR_NUM;
+			last_type = type = ZBX_EVAL_TOKEN_VAR_NUM;
 			offset++;
 		}
 		else if (SUCCEED == eval_is_compound_number_char(ctx->expression[offset], offset - pos))
@@ -384,6 +386,15 @@ static void	eval_parse_greater_character_token(zbx_eval_context_t *ctx, size_t p
 		eval_parse_character_token(pos, ZBX_EVAL_TOKEN_OP_GT, token);
 }
 
+static zbx_eval_token_t	*eval_get_last_function_token(zbx_eval_context_t *ctx)
+{
+	for(int i = ctx->ops.values_num - 1; i >= 0; i --)
+		if (0 != (ctx->ops.values[i].type & ZBX_EVAL_CLASS_FUNCTION))
+			return &ctx->ops.values[i];
+
+	return NULL;
+}
+
 /******************************************************************************
  *                                                                            *
  * Purpose: parse string variable token                                       *
@@ -401,7 +412,14 @@ static void	eval_parse_greater_character_token(zbx_eval_context_t *ctx, size_t p
  ******************************************************************************/
 static int	eval_parse_string_token(zbx_eval_context_t *ctx, size_t pos, zbx_eval_token_t *token, char **error)
 {
-	const char	*ptr = ctx->expression + pos + 1;
+	const char		*ptr = ctx->expression + pos + 1;
+	int			is_history_function;
+	zbx_eval_token_t	*func_token;
+	char			*value = NULL;
+
+	/* workaround for calculated items is required due to present bug */
+	func_token = eval_get_last_function_token(ctx);
+	is_history_function = NULL != func_token && ZBX_EVAL_TOKEN_HIST_FUNCTION == func_token->type;
 
 	for (; '\0' != *ptr; ptr++)
 	{
@@ -411,22 +429,40 @@ static int	eval_parse_string_token(zbx_eval_context_t *ctx, size_t pos, zbx_eval
 			token->loc.l = pos;
 			token->loc.r = ptr - ctx->expression;
 			eval_update_const_variable(ctx, token);
+
+			if (0 != is_history_function)
+			{
+				value = zbx_substr_unquote_opt(ctx->expression,
+						token->loc.l, token->loc.r, ZBX_STRQUOTE_SKIP_BACKSLASH);
+
+				if (NULL != value)
+					zbx_variant_set_str(&token->value, value);
+			}
+
 			return SUCCEED;
 		}
 
 		if ('\\' == *ptr)
 		{
-			if ('"' != ptr[1] && '\\' != ptr[1] )
+			if (0 != is_history_function)
 			{
-				*error = zbx_dsprintf(*error, "invalid escape sequence in string starting with \"%s\"",
-						ptr);
+				if ('"' == ptr[1])
+					ptr++;
+				continue;
+			}
+
+			if ('"' != ptr[1] && '\\' != ptr[1])
+			{
+				*error = zbx_dsprintf(*error,
+						"invalid escape sequence in string starting with \"%s\"", ptr);
+
 				return FAIL;
 			}
 			ptr++;
 		}
 	}
-
 	*error = zbx_dsprintf(*error, "unterminated string at \"%s\"", ctx->expression + pos);
+
 	return FAIL;
 }
 
@@ -608,6 +644,7 @@ static int	eval_parse_query_filter(const char **ptr)
 size_t	eval_parse_query(const char *str, const char **phost, const char **pkey, const char **pfilter)
 {
 #define MVAR_HOST_HOST	"{HOST.HOST"
+#define MVAR_ITEM_KEY	"{ITEM.KEY"
 
 	const char	*host = str + 1, *key, *filter, *end;
 
@@ -621,7 +658,7 @@ size_t	eval_parse_query(const char *str, const char **phost, const char **pkey,
 	{
 		if (0 == strncmp(key, MVAR_HOST_HOST, ZBX_CONST_STRLEN(MVAR_HOST_HOST)))
 		{
-			int	offset = 0;
+			size_t	offset = 0;
 
 			if ('}' == key[ZBX_CONST_STRLEN(MVAR_HOST_HOST)])
 			{
@@ -649,7 +686,29 @@ size_t	eval_parse_query(const char *str, const char **phost, const char **pkey,
 	end = ++key;
 
 	if ('*' == *key)
+	{
 		end++;
+	}
+	else if ('{' == *key)
+	{
+		if (0 == strncmp(key, MVAR_ITEM_KEY, ZBX_CONST_STRLEN(MVAR_ITEM_KEY)))
+		{
+			size_t	offset = 0;
+
+			if ('}' == key[ZBX_CONST_STRLEN(MVAR_ITEM_KEY)])
+			{
+				offset = 1;
+			}
+			else if (0 != isdigit((unsigned char)key[ZBX_CONST_STRLEN(MVAR_ITEM_KEY)]) &&
+					'}' == key[ZBX_CONST_STRLEN(MVAR_ITEM_KEY) + 1])
+			{
+				offset = 2;
+			}
+
+			if (0 != offset)
+				end += ZBX_CONST_STRLEN(MVAR_ITEM_KEY) + offset;
+		}
+	}
 	else if (SUCCEED != parse_key(&end))
 		return 0;
 
@@ -673,6 +732,7 @@ size_t	eval_parse_query(const char *str, const char **phost, const char **pkey,
 	return end - str;
 
 #undef MVAR_HOST_HOST
+#undef MVAR_ITEM_KEY
 }
 
 /******************************************************************************
@@ -932,7 +992,7 @@ static int	eval_parse_token(zbx_eval_context_t *ctx, size_t pos, zbx_eval_token_
 		case '8':
 		case '9':
 			/* after ',' there will be at least one value on the stack */
-			if (ZBX_EVAL_TOKEN_COMMA == ctx->last_token_type &&
+			if (ZBX_EVAL_TOKEN_COMMA == ctx->last_token_type && 0 < ctx->stack.values_num &&
 				ZBX_EVAL_TOKEN_ARG_QUERY == ctx->stack.values[ctx->stack.values_num - 1].type)
 			{
 				return eval_parse_period_token(ctx, pos, token, error);
@@ -1321,6 +1381,12 @@ static int	eval_parse_expression(zbx_eval_context_t *ctx, const char *expression
 						goto out;
 					ctx->ops.values_num--;
 				}
+				else if (ZBX_EVAL_TOKEN_GROUP_OPEN == ctx->last_token_type)
+				{
+					*error = zbx_dsprintf(*error, "parenthesis cannot close empty group at \"%s\"",
+							ctx->expression + pos);
+					goto out;
+				}
 			}
 		}
 		else if (0 != (token.type & (ZBX_EVAL_CLASS_OPERAND | ZBX_EVAL_CLASS_PROPERTY)))
