@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,27 +21,31 @@
 
 #include "common.h"
 #include "json.h"
-
-static zbx_int64_t	json_parse_object(const char *start, char **error);
+#include "jsonobj.h"
 
 /******************************************************************************
  *                                                                            *
  * Purpose: Prepares JSON parsing error message                               *
  *                                                                            *
- * Parameters: message     - [IN] the error message                           *
- *             json_buffer - [IN] the failing data fragment                   *
- *             error       - [OUT] the parsing error message (can be NULL)    *
+ * Parameters: message - [IN] the error message                               *
+ *             ptr     - [IN] the failing data fragment                       *
+ *             error   - [OUT] the parsing error message (can be NULL)        *
  *                                                                            *
  * Return value: 0 - the json_error() function always returns 0 value         *
  *                      so it can be used to return from failed parses        *
  *                                                                            *
  ******************************************************************************/
-static zbx_int64_t	json_error(const char *message, const char *json_buffer, char **error)
+zbx_int64_t	json_error(const char *message, const char *ptr, char **error)
 {
 	if (NULL != error)
 	{
-		if (NULL != json_buffer)
-			*error = zbx_dsprintf(*error, "%s at: '%s'", message, json_buffer);
+		if (NULL != ptr)
+		{
+			if (128 < strlen(ptr))
+				*error = zbx_dsprintf(*error, "%s at: '%128s...'", message, ptr);
+			else
+				*error = zbx_dsprintf(*error, "%s at: '%s'", message, ptr);
+		}
 		else
 			*error = zbx_strdup(*error, message);
 	}
@@ -54,6 +58,7 @@ static zbx_int64_t	json_error(const char *message, const char *json_buffer, char
  * Purpose: Parses JSON string value or object name                           *
  *                                                                            *
  * Parameters: start - [IN] the JSON data without leading whitespace          *
+ *             str   - [OUT] the parsed unquoted string (can be NULL)         *
  *             error - [OUT] the parsing error message (can be NULL)          *
  *                                                                            *
  * Return value: The number of characters parsed. On error 0 is returned and  *
@@ -61,7 +66,7 @@ static zbx_int64_t	json_error(const char *message, const char *json_buffer, char
  *               message.                                                     *
  *                                                                            *
  ******************************************************************************/
-static zbx_int64_t	json_parse_string(const char *start, char **error)
+static zbx_int64_t	json_parse_string(const char *start, char **str, char **error)
 {
 	const char	*ptr = start;
 
@@ -77,7 +82,7 @@ static zbx_int64_t	json_parse_string(const char *start, char **error)
 		if ('\\' == *ptr)
 		{
 			const char	*escape_start = ptr;
-			int		i;
+			unsigned char	uc[4];	/* decoded Unicode character takes 1-4 bytes in UTF-8 */
 
 			/* unexpected end of string data, failing */
 			if ('\0' == *(++ptr))
@@ -96,16 +101,13 @@ static zbx_int64_t	json_parse_string(const char *start, char **error)
 					break;
 				case 'u':
 					/* check if the \u is followed with 4 hex digits */
-					for (i = 0; i < 4; i++)
+					if (0 == zbx_json_decode_character(&ptr, uc))
 					{
-						if (0 == isxdigit((unsigned char)*(++ptr)))
-						{
-							return json_error("invalid escape sequence in string",
-									escape_start, error);
-						}
+						return json_error("invalid escape sequence in string",
+								escape_start, error);
 					}
 
-					break;
+					continue;
 				default:
 					return json_error("invalid escape sequence in string data",
 							escape_start, error);
@@ -119,6 +121,17 @@ static zbx_int64_t	json_parse_string(const char *start, char **error)
 		ptr++;
 	}
 
+	if (NULL != str)
+	{
+		*str = (char *)zbx_malloc(NULL, (size_t)(ptr - start));
+
+		if (NULL == json_copy_string(start, *str, (size_t)(ptr - start)))
+		{
+			zbx_free(*str);
+			return json_error("invalid string data", start, error);
+		}
+	}
+
 	return ptr - start + 1;
 }
 
@@ -127,6 +140,8 @@ static zbx_int64_t	json_parse_string(const char *start, char **error)
  * Purpose: Parses JSON array value                                           *
  *                                                                            *
  * Parameters: start - [IN] the JSON data without leading whitespace          *
+ *             obj   - [IN/OUT] the JSON object (can be NULL)                 *
+ *             depth - [IN]                                                   *
  *             error - [OUT] the parsing error message (can be NULL)          *
  *                                                                            *
  * Return value: The number of characters parsed. On error 0 is returned and  *
@@ -134,11 +149,14 @@ static zbx_int64_t	json_parse_string(const char *start, char **error)
  *               message.                                                     *
  *                                                                            *
  ******************************************************************************/
-static zbx_int64_t	json_parse_array(const char *start, char **error)
+zbx_int64_t	json_parse_array(const char *start, zbx_jsonobj_t *obj, int depth, char **error)
 {
 	const char	*ptr = start;
 	zbx_int64_t	len;
 
+	if (NULL != obj)
+		jsonobj_init(obj, ZBX_JSON_TYPE_ARRAY);
+
 	ptr++;
 	SKIP_WHITESPACE(ptr);
 
@@ -146,9 +164,29 @@ static zbx_int64_t	json_parse_array(const char *start, char **error)
 	{
 		while (1)
 		{
+			zbx_jsonobj_t	*value;
+
+			if (NULL != obj)
+			{
+				value = zbx_malloc(NULL, sizeof(zbx_jsonobj_t));
+				jsonobj_init(value, ZBX_JSON_TYPE_UNKNOWN);
+			}
+			else
+				value = NULL;
+
 			/* json_parse_value strips leading whitespace, so we don't have to do it here */
-			if (0 == (len = json_parse_value(ptr, error)))
+			if (0 == (len = json_parse_value(ptr, value, depth, error)))
+			{
+				if (NULL != obj)
+				{
+					zbx_jsonobj_clear(value);
+					zbx_free(value);
+				}
 				return 0;
+			}
+
+			if (NULL != obj)
+				zbx_vector_jsonobj_ptr_append(&obj->data.array, value);
 
 			ptr += len;
 			SKIP_WHITESPACE(ptr);
@@ -171,15 +209,16 @@ static zbx_int64_t	json_parse_array(const char *start, char **error)
  *                                                                            *
  * Purpose: Parses JSON number value                                          *
  *                                                                            *
- * Parameters: start - [IN] the JSON data without leading whitespace          *
- *             error - [OUT] the parsing error message (can be NULL)          *
+ * Parameters: start  - [IN] the JSON data without leading whitespace         *
+ *             number - [OUT] the parsed number (can be NULL)                 *
+ *             error  - [OUT] the parsing error message (can be NULL)         *
  *                                                                            *
  * Return value: The number of characters parsed. On error 0 is returned and  *
  *               error parameter (if not NULL) contains allocated error       *
  *               message.                                                     *
  *                                                                            *
  ******************************************************************************/
-static zbx_int64_t	json_parse_number(const char *start, char **error)
+static zbx_int64_t	json_parse_number(const char *start, double *number, char **error)
 {
 	const char	*ptr = start;
 	char		first_digit;
@@ -235,6 +274,9 @@ static zbx_int64_t	json_parse_number(const char *start, char **error)
 		}
 	}
 
+	if (NULL != number)
+		*number = atof(start);
+
 	return ptr - start;
 }
 
@@ -274,6 +316,8 @@ static zbx_int64_t	json_parse_literal(const char *start, const char *text, char
  * Purpose: Parses JSON object value                                          *
  *                                                                            *
  * Parameters: start - [IN] the JSON data                                     *
+ *             obj   - [IN/OUT] JSON object (can be NULL)                     *
+ *             depth - [IN]                                                   *
  *             error - [OUT] the parsing error message (can be NULL)          *
  *                                                                            *
  * Return value: The number of characters parsed. On error 0 is returned and  *
@@ -281,10 +325,23 @@ static zbx_int64_t	json_parse_literal(const char *start, const char *text, char
  *               message.                                                     *
  *                                                                            *
  ******************************************************************************/
-zbx_int64_t	json_parse_value(const char *start, char **error)
+zbx_int64_t	json_parse_value(const char *start, zbx_jsonobj_t *obj, int depth, char **error)
 {
+#define ZBX_MAX_JSON_DEPTH	64
 	const char	*ptr = start;
 	zbx_int64_t	len;
+	char		*str = NULL;
+	double		number;
+
+	if (ZBX_MAX_JSON_DEPTH < depth)
+	{
+		char	err_buf[MAX_STRING_LEN];
+
+		zbx_snprintf(err_buf, sizeof(err_buf), "JSON depth exceeds %d", ZBX_MAX_JSON_DEPTH);
+		return json_error(err_buf, ptr, error);
+	}
+
+	depth++;
 
 	SKIP_WHITESPACE(ptr);
 
@@ -293,28 +350,40 @@ zbx_int64_t	json_parse_value(const char *start, char **error)
 		case '\0':
 			return json_error("unexpected end of object value", NULL, error);
 		case '"':
-			if (0 == (len = json_parse_string(ptr, error)))
+			if (0 == (len = json_parse_string(ptr, (NULL != obj ? &str : NULL), error)))
 				return 0;
+
+			if (NULL != obj)
+				jsonobj_set_string(obj, str);
 			break;
 		case '{':
-			if (0 == (len = json_parse_object(ptr, error)))
+			if (0 == (len = json_parse_object(ptr, obj, depth, error)))
 				return 0;
 			break;
 		case '[':
-			if (0 == (len = json_parse_array(ptr, error)))
+			if (0 == (len = json_parse_array(ptr, obj, depth, error)))
 				return 0;
 			break;
 		case 't':
 			if (0 == (len = json_parse_literal(ptr, "true", error)))
 				return 0;
+
+			if (NULL != obj)
+				jsonobj_set_true(obj);
 			break;
 		case 'f':
 			if (0 == (len = json_parse_literal(ptr, "false", error)))
 				return 0;
+
+			if (NULL != obj)
+				jsonobj_set_false(obj);
 			break;
 		case 'n':
 			if (0 == (len = json_parse_literal(ptr, "null", error)))
 				return 0;
+
+			if (NULL != obj)
+				jsonobj_set_null(obj);
 			break;
 		case '0':
 		case '1':
@@ -327,14 +396,19 @@ zbx_int64_t	json_parse_value(const char *start, char **error)
 		case '8':
 		case '9':
 		case '-':
-			if (0 == (len = json_parse_number(ptr, error)))
+			if (0 == (len = json_parse_number(ptr, (NULL != obj ? &number : NULL), error)))
 				return 0;
+
+			if (NULL != obj)
+				jsonobj_set_number(obj, number);
+
 			break;
 		default:
 			return json_error("invalid JSON object value starting character", ptr, error);
 	}
 
 	return ptr - start + len;
+#undef ZBX_MAX_JSON_DEPTH
 }
 
 /******************************************************************************
@@ -342,6 +416,8 @@ zbx_int64_t	json_parse_value(const char *start, char **error)
  * Purpose: Parses JSON object                                                *
  *                                                                            *
  * Parameters: start - [IN] the JSON data                                     *
+ *             obj   - [IN/OUT] the JSON object (can be NULL)                 *
+ *             depth - [IN]                                                   *
  *             error - [OUT] the parsing error message (can be NULL)          *
  *                                                                            *
  * Return value: The number of characters parsed. On error 0 is returned and  *
@@ -349,10 +425,13 @@ zbx_int64_t	json_parse_value(const char *start, char **error)
  *               message.                                                     *
  *                                                                            *
  ******************************************************************************/
-static zbx_int64_t	json_parse_object(const char *start, char **error)
+zbx_int64_t	json_parse_object(const char *start, zbx_jsonobj_t *obj, int depth, char **error)
 {
-	const char	*ptr = start;
-	zbx_int64_t	len;
+	const char		*ptr = start;
+	zbx_int64_t		len;
+
+	if (NULL != obj)
+		jsonobj_init(obj, ZBX_JSON_TYPE_OBJECT);
 
 	/* parse object name */
 	SKIP_WHITESPACE(ptr);
@@ -364,11 +443,15 @@ static zbx_int64_t	json_parse_object(const char *start, char **error)
 	{
 		while (1)
 		{
+			zbx_jsonobj_el_t	el;
+
 			if ('"' != *ptr)
 				return json_error("invalid object name", ptr, error);
 
+			jsonobj_el_init(&el);
+
 			/* cannot parse object name, failing */
-			if (0 == (len = json_parse_string(ptr, error)))
+			if (0 == (len = json_parse_string(ptr, (NULL != obj ? &el.name : NULL), error)))
 				return 0;
 
 			ptr += len;
@@ -377,11 +460,34 @@ static zbx_int64_t	json_parse_object(const char *start, char **error)
 			SKIP_WHITESPACE(ptr);
 
 			if (':' != *ptr)
+			{
+				jsonobj_el_clear(&el);
 				return json_error("invalid object name/value separator", ptr, error);
+			}
+
 			ptr++;
 
-			if (0 == (len = json_parse_value(ptr, error)))
+			if (0 == (len = json_parse_value(ptr, (NULL != obj ? &el.value : NULL), depth, error)))
+			{
+				jsonobj_el_clear(&el);
 				return 0;
+			}
+
+			if (NULL != obj)
+			{
+				zbx_jsonobj_el_t	*pel;
+
+				pel = (zbx_jsonobj_el_t *)zbx_hashset_insert(&obj->data.object, &el, sizeof(el));
+
+				/* check if they element was inserted, if not solve the conflict */
+				/* by overwriting old data                                       */
+				if (pel->name != el.name)
+				{
+					zbx_free(pel->name);
+					zbx_jsonobj_clear(&pel->value);
+					*pel = el;
+				}
+			}
 
 			ptr += len;
 
@@ -426,11 +532,11 @@ zbx_int64_t	zbx_json_validate(const char *start, char **error)
 	switch (*start)
 	{
 		case '{':
-			if (0 == (len = json_parse_object(start, error)))
+			if (0 == (len = json_parse_object(start, NULL, 0, error)))
 				return 0;
 			break;
 		case '[':
-			if (0 == (len = json_parse_array(start, error)))
+			if (0 == (len = json_parse_array(start, NULL, 0, error)))
 				return 0;
 			break;
 		default:
