@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,13 +26,13 @@
 #include "log.h"
 #include "dir.h"
 #include "sha256crypt.h"
+#include "zbxalgo.h"
 
 #if defined(_WINDOWS) || defined(__MINGW32__)
 #include "aclapi.h"
 #include "sddl.h"
 #endif
 
-#define ZBX_MAX_DB_FILE_SIZE	64 * ZBX_KIBIBYTE	/* files larger than 64 KB cannot be stored in the database */
 
 extern int	CONFIG_TIMEOUT;
 
@@ -352,6 +352,7 @@ int	VFS_FILE_EXISTS(AGENT_REQUEST *request, AGENT_RESULT *result)
 
 int	VFS_FILE_CONTENTS(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
+#define ZBX_MAX_VFS_FILE_SIZE	16 * ZBX_MEBIBYTE	/* file contents larger than 64 KB will be truncated in the database */
 	char		*filename, *tmp, encoding[32];
 	char		read_buf[MAX_BUFFER_LEN], *utf8, *contents = NULL;
 	size_t		contents_alloc = 0, contents_offset = 0;
@@ -399,7 +400,7 @@ int	VFS_FILE_CONTENTS(AGENT_REQUEST *request, AGENT_RESULT *result)
 		goto err;
 	}
 
-	if (ZBX_MAX_DB_FILE_SIZE < stat_buf.st_size)
+	if (ZBX_MAX_VFS_FILE_SIZE < stat_buf.st_size)
 	{
 		SET_MSG_RESULT(result, zbx_strdup(NULL, "File is too large for this check."));
 		goto err;
@@ -413,7 +414,7 @@ int	VFS_FILE_CONTENTS(AGENT_REQUEST *request, AGENT_RESULT *result)
 
 	flen = 0;
 
-	while (0 < (nbytes = zbx_read(f, read_buf, sizeof(read_buf), encoding)))
+	while (0 < (nbytes = read(f, read_buf, sizeof(read_buf))))
 	{
 		if (CONFIG_TIMEOUT < zbx_time() - ts)
 		{
@@ -422,16 +423,14 @@ int	VFS_FILE_CONTENTS(AGENT_REQUEST *request, AGENT_RESULT *result)
 			goto err;
 		}
 
-		if (ZBX_MAX_DB_FILE_SIZE < (flen += nbytes))
+		if (ZBX_MAX_VFS_FILE_SIZE < (flen += nbytes))
 		{
 			SET_MSG_RESULT(result, zbx_strdup(NULL, "File is too large for this check."));
 			zbx_free(contents);
 			goto err;
 		}
 
-		utf8 = convert_to_utf8(read_buf, nbytes, encoding);
-		zbx_strcpy_alloc(&contents, &contents_alloc, &contents_offset, utf8);
-		zbx_free(utf8);
+		zbx_str_memcpy_alloc(&contents, &contents_alloc, &contents_offset, read_buf, nbytes);
 	}
 
 	if (-1 == nbytes)	/* error occurred */
@@ -441,16 +440,16 @@ int	VFS_FILE_CONTENTS(AGENT_REQUEST *request, AGENT_RESULT *result)
 		goto err;
 	}
 
-	if (0 != contents_offset)
-		contents_offset -= zbx_rtrim(contents, "\r\n");
-
-	if (0 == contents_offset) /* empty file */
+	if (NULL != contents)
 	{
+		utf8 = convert_to_utf8(contents, contents_offset, encoding);
 		zbx_free(contents);
-		contents = zbx_strdup(contents, "");
-	}
+		zbx_rtrim_utf8(utf8, "\r\n");
 
-	SET_TEXT_RESULT(result, contents);
+		SET_TEXT_RESULT(result, utf8);
+	}
+	else
+		SET_TEXT_RESULT(result, zbx_strdup(NULL, ""));
 
 	ret = SYSINFO_RET_OK;
 err:
@@ -458,6 +457,7 @@ err:
 		close(f);
 
 	return ret;
+#undef ZBX_MAX_VFS_FILE_SIZE
 }
 
 int	VFS_FILE_REGEXP(AGENT_REQUEST *request, AGENT_RESULT *result)
@@ -534,7 +534,7 @@ int	VFS_FILE_REGEXP(AGENT_REQUEST *request, AGENT_RESULT *result)
 		goto err;
 	}
 
-	while (0 < (nbytes = zbx_read(f, buf, sizeof(buf), encoding)))
+	while (0 < (nbytes = zbx_read_text_line_from_file(f, buf, sizeof(buf), encoding)))
 	{
 		if (CONFIG_TIMEOUT < zbx_time() - ts)
 		{
@@ -564,7 +564,12 @@ int	VFS_FILE_REGEXP(AGENT_REQUEST *request, AGENT_RESULT *result)
 		}
 	}
 
-	if (-1 == nbytes)	/* error occurred */
+	if (ZBX_READ_WRONG_ENCODING == nbytes)
+	{
+		SET_MSG_RESULT(result, zbx_strdup(NULL, "Cannot read from file. Wrong encoding detected."));
+		goto err;
+	}
+	else if (ZBX_READ_ERR == nbytes)	/* general error occurred */
 	{
 		SET_MSG_RESULT(result, zbx_strdup(NULL, "Cannot read from file."));
 		goto err;
@@ -656,7 +661,7 @@ int	VFS_FILE_REGMATCH(AGENT_REQUEST *request, AGENT_RESULT *result)
 
 	res = 0;
 
-	while (0 == res && 0 < (nbytes = zbx_read(f, buf, sizeof(buf), encoding)))
+	while (0 == res && 0 < (nbytes = zbx_read_text_line_from_file(f, buf, sizeof(buf), encoding)))
 	{
 		if (CONFIG_TIMEOUT < zbx_time() - ts)
 		{
@@ -677,7 +682,12 @@ int	VFS_FILE_REGMATCH(AGENT_REQUEST *request, AGENT_RESULT *result)
 			break;
 	}
 
-	if (-1 == nbytes)	/* error occurred */
+	if (ZBX_READ_WRONG_ENCODING == nbytes)
+	{
+		SET_MSG_RESULT(result, zbx_strdup(NULL, "Cannot read from file. Wrong encoding detected."));
+		goto err;
+	}
+	else if (ZBX_READ_ERR == nbytes)	/* error occurred */
 	{
 		SET_MSG_RESULT(result, zbx_strdup(NULL, "Cannot read from file."));
 		goto err;
@@ -1269,6 +1279,69 @@ static char	*get_print_time(time_t st_raw)
 #	define ZBX_DIR_DELIMITER	"\\"
 #else
 #	define ZBX_DIR_DELIMITER	"/"
+
+static char	*canonicalize_path(const char *fullname)
+{
+	int			i, up_level = 0;
+	char			*name;
+	const char		*p_start = &fullname[1], *p_to_delimiter;
+	size_t			name_alloc = 0, name_offset = 0;
+	zbx_vector_str_t	names;
+
+	zbx_vector_str_create(&names);
+
+	do
+	{
+		if (NULL != (p_to_delimiter = strchr(p_start, '/')))
+		{
+			name = zbx_dsprintf(NULL, "%.*s", (int)(p_to_delimiter - p_start), p_start);
+			p_start = p_to_delimiter + 1;
+		}
+		else
+			name = zbx_strdup(NULL, p_start);
+
+		zbx_vector_str_append(&names, name);
+	}
+	while (NULL != p_to_delimiter);
+
+	name = NULL;
+
+	for (i = names.values_num - 1; 0 <= i; i--)
+	{
+		char *ptr = names.values[i];
+
+		if (0 == strcmp(ptr, ".") || 0 == strlen(ptr))
+		{
+			zbx_free(ptr);
+			zbx_vector_str_remove(&names, i);
+		}
+		else if (0 == strcmp(ptr, ".."))
+		{
+			zbx_free(ptr);
+			zbx_vector_str_remove(&names, i);
+			up_level++;
+		}
+		else if (0 < up_level)
+		{
+			zbx_free(ptr);
+			zbx_vector_str_remove(&names, i);
+			up_level--;
+		}
+	}
+
+	if (0 < names.values_num)
+	{
+		for (i = 0; i < names.values_num; i++)
+			zbx_snprintf_alloc(&name, &name_alloc, &name_offset, "/%s", names.values[i]);
+	}
+	else
+		zbx_snprintf_alloc(&name, &name_alloc, &name_offset, "/");
+
+	zbx_vector_str_clear_ext(&names, zbx_str_free);
+	zbx_vector_str_destroy(&names);
+
+	return name;
+}
 #endif
 
 static int	get_dir_names(const char *filename, char **basename, char **dirname, char **pathname)
@@ -1279,16 +1352,36 @@ static int	get_dir_names(const char *filename, char **basename, char **dirname,
 #if defined(_WINDOWS) || defined(__MINGW32__)
 	if (NULL == (*pathname = _fullpath(NULL, filename, 0)))
 		return FAIL;
-#elif defined(__hpux)
-	char resolved_path[PATH_MAX + 1];
-
-	if (NULL == (*pathname = realpath(filename, resolved_path)))
-		return FAIL;
-
-	*pathname = zbx_strdup(NULL, *pathname);
 #else
-	if (NULL == (*pathname = realpath(filename, NULL)))
-		return FAIL;
+	if ( '/' != filename[0])
+	{
+#ifdef PATH_MAX
+#	define MAX_PATH_BUFFER	PATH_MAX
+#else
+#	define MAX_PATH_BUFFER	4096
+#endif
+		char	resolved_path[MAX_PATH_BUFFER + 1], *name = NULL;
+		size_t	name_alloc = 0, name_offset = 0;
+
+#define ZBX_UNREACHABLE_STR		"(unreachable)"
+#define ZBX_UNREACHABLE_STR_LEN		ZBX_CONST_STRLEN(ZBX_UNREACHABLE_STR)
+
+		if (NULL == getcwd(resolved_path, MAX_PATH_BUFFER) || 0 == strncmp(ZBX_UNREACHABLE_STR, resolved_path,
+				ZBX_UNREACHABLE_STR_LEN))
+		{
+			return FAIL;
+		}
+
+		zbx_snprintf_alloc(&name, &name_alloc, &name_offset, "%s/%s", resolved_path, filename);
+
+		*pathname = canonicalize_path(name);
+		zbx_free(name);
+#undef ZBX_UNREACHABLE_STR_LEN
+#undef ZBX_UNREACHABLE_STR
+#undef MAX_PATH_BUFFER
+	}
+	else
+		*pathname = canonicalize_path(filename);
 #endif
 
 	ptr1 = *pathname;
