@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -1053,6 +1053,7 @@ static int	process_value(zbx_vector_ptr_t *addrs, zbx_vector_ptr_t *agent2_resul
 		el->logeventid = (int)*logeventid;
 
 	zbx_timespec(&el->ts);
+
 	el->flags = flags;
 	el->id = ++last_valueid;
 
@@ -1229,7 +1230,7 @@ static void	process_active_checks(zbx_vector_ptr_t *addrs)
 		else if (0 != ((ZBX_METRIC_FLAG_LOG_LOG | ZBX_METRIC_FLAG_LOG_LOGRT) & metric->flags))
 		{
 			ret = process_log_check(addrs, NULL, &regexps, metric, process_value, &lastlogsize_sent,
-					&mtime_sent, &error, &pre_persistent_vec);
+					&mtime_sent, &error, &pre_persistent_vec, 0);
 		}
 		else if (0 != (ZBX_METRIC_FLAG_LOG_EVENTLOG & metric->flags))
 		{
@@ -1397,7 +1398,7 @@ ZBX_THREAD_ENTRY(active_checks_thread, args)
 		}
 #endif
 
-		zbx_update_env(zbx_time());
+		zbx_update_env(get_process_type_string(process_type), zbx_time());
 
 		if ((now = time(NULL)) >= nextsend)
 		{
@@ -1458,7 +1459,8 @@ ZBX_THREAD_ENTRY(active_checks_thread, args)
 	zbx_free(session_token);
 
 #ifdef _WINDOWS
-	zbx_addr_free(&activechk_args.addrs);
+	zbx_vector_ptr_clear_ext(&activechk_args.addrs, (zbx_clean_func_t)zbx_addr_free);
+	zbx_vector_ptr_destroy(&activechk_args.addrs);
 	free_active_metrics();
 
 	ZBX_DO_EXIT();
