@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -79,7 +79,7 @@ static void	deactivate_perf_counter(zbx_perf_counter_data_t *counter)
  *           added, a pointer to that counter is returned, NULL otherwise     *
  *                                                                            *
  ******************************************************************************/
-zbx_perf_counter_data_t	*add_perf_counter(const char *name, const char *counterpath, int interval,
+static zbx_perf_counter_data_t	*add_perf_counter(const char *name, const char *counterpath, int interval,
 		zbx_perf_counter_lang_t lang, char **error)
 {
 	zbx_perf_counter_data_t	*cptr = NULL;
@@ -88,8 +88,6 @@ zbx_perf_counter_data_t	*add_perf_counter(const char *name, const char *counterp
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s() counter:'%s' interval:%d", __func__, counterpath, interval);
 
-	LOCK_PERFCOUNTERS;
-
 	if (SUCCEED != perf_collector_started())
 	{
 		*error = zbx_strdup(*error, "Performance collector is not started.");
@@ -155,13 +153,25 @@ zbx_perf_counter_data_t	*add_perf_counter(const char *name, const char *counterp
 		zbx_free(alias_name);
 	}
 out:
-	UNLOCK_PERFCOUNTERS;
-
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s(): %s", __func__, NULL == cptr ? "FAIL" : "SUCCEED");
 
 	return cptr;
 }
 
+zbx_perf_counter_data_t	*zbx_add_perf_counter(const char *name, const char *counterpath, int interval,
+		zbx_perf_counter_lang_t lang, char **error)
+{
+	zbx_perf_counter_data_t	*v;
+
+	LOCK_PERFCOUNTERS;
+
+	v = add_perf_counter(name, counterpath, interval, lang, error);
+
+	UNLOCK_PERFCOUNTERS;
+
+	return v;
+}
+
 /******************************************************************************
  *                                                                            *
  * Purpose: extends the performance counter buffer to store the new data      *
@@ -340,7 +350,7 @@ out:
  *           the memory is freed - do not use it again                        *
  *                                                                            *
  ******************************************************************************/
-void	remove_perf_counter(zbx_perf_counter_data_t *counter)
+void	zbx_remove_perf_counter(zbx_perf_counter_data_t *counter)
 {
 	zbx_perf_counter_data_t	*cptr;
 
@@ -455,10 +465,7 @@ int	init_perf_collector(zbx_threadedness_t threadedness, char **error)
 	}
 
 	if (SUCCEED != set_object_names())
-	{
-		*error = zbx_strdup(*error, "cannot initialize object names");
-		goto out;
-	}
+		zabbix_log(LOG_LEVEL_WARNING, "%s(): cannot initialize object names", __func__);
 
 	ret = SUCCEED;
 out:
@@ -678,8 +685,6 @@ int	get_perf_counter_value_by_name(const char *name, double *value, char **error
 
 	counterpath = zbx_strdup(counterpath, perfs->counterpath);
 out:
-	UNLOCK_PERFCOUNTERS;
-
 	if (NULL != counterpath)
 	{
 		/* request counter value directly from Windows performance counters */
@@ -693,6 +698,8 @@ out:
 		zbx_free(counterpath);
 	}
 
+	UNLOCK_PERFCOUNTERS;
+
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __func__, zbx_result_string(ret));
 
 	return ret;
@@ -754,8 +761,6 @@ int	get_perf_counter_value_by_path(const char *counterpath, int interval, zbx_pe
 	if (NULL == perfs)
 		perfs = add_perf_counter(NULL, counterpath, interval, lang, error);
 out:
-	UNLOCK_PERFCOUNTERS;
-
 	if (SUCCEED != ret && NULL != perfs)
 	{
 		/* request counter value directly from Windows performance counters */
@@ -763,6 +768,8 @@ out:
 			ret = SUCCEED;
 	}
 
+	UNLOCK_PERFCOUNTERS;
+
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __func__, zbx_result_string(ret));
 
 	return ret;
