@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 
 #include "dbcache.h"
 #include "export.h"
+#include "zbxprof.h"
 
 extern int				CONFIG_HISTSYNCER_FREQUENCY;
 extern ZBX_THREAD_LOCAL unsigned char	process_type;
@@ -49,7 +50,8 @@ static void	zbx_db_flush_timer_queue(void)
 
 	if (0 != persistent_timers.values_num)
 	{
-		zbx_db_insert_prepare(&db_insert, "trigger_queue", "trigger_queueid", "objectid", "type", "clock", "ns", NULL);
+		zbx_db_insert_prepare(&db_insert, "trigger_queue", "trigger_queueid", "objectid", "type", "clock", "ns",
+				(char *)NULL);
 
 		for (i = 0; i < persistent_timers.values_num; i++)
 		{
@@ -129,7 +131,8 @@ ZBX_THREAD_ENTRY(dbsyncer_thread, args)
 	for (;;)
 	{
 		sec = zbx_time();
-		zbx_update_env(sec);
+
+		zbx_prof_update(get_process_type_string(process_type), sec);
 
 		if (0 != sleeptime)
 			zbx_setproctitle("%s #%d [%s, syncing history]", process_name, process_num, stats);
@@ -140,7 +143,9 @@ ZBX_THREAD_ENTRY(dbsyncer_thread, args)
 
 		/* database APIs might not handle signals correctly and hang, block signals to avoid hanging */
 		zbx_block_signals(&orig_mask);
+		zbx_prof_start(__func__, ZBX_PROF_PROCESSING);
 		zbx_sync_history_cache(&values_num, &triggers_num, &more);
+		zbx_prof_end();
 
 		if (!ZBX_IS_RUNNING() && SUCCEED != zbx_db_trigger_queue_locked())
 			zbx_db_flush_timer_queue();
